package com.persagy.apm.report.dependencies.energyefficiencyweb.service;

import com.alibaba.fastjson.JSON;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.energyefficiencyweb.client.EnergyEfficiencyClient;
import com.persagy.apm.report.dependencies.energyefficiencyweb.enums.EnergyEfficiencyTimeTypeEnum;
import com.persagy.apm.report.dependencies.energyefficiencyweb.model.dto.QueryEnergyEfficiencyDataDTO;
import com.persagy.apm.report.dependencies.energyefficiencyweb.model.vo.EnergyEfficiencyDataListVO;
import com.persagy.apm.report.dependencies.energyefficiencyweb.model.vo.EnergyEfficiencyDataVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 能效情况service类
 *
 * @author lixing
 * @version V1.0 2021/7/28 5:08 下午
 **/
@Service
public class EnergyEfficiencyServiceImpl {
    @Autowired
    private EnergyEfficiencyClient energyEfficiencyClient;

    /**
     * 查询月能效
     *
     * @param targetId  能效指标id
     * @param projectId 项目id
     * @param date      日期
     * @return 能效值
     * @author lixing
     * @version V1.0 2021/7/28 5:09 下午
     */
    public String getMonthlyEnergyEfficiency(String targetId, String projectId, Date date) {
        // 从能效系统查询数据
        QueryEnergyEfficiencyDataDTO queryEnergyEfficiencyDataDTO = initQueryDTO(targetId, projectId, date);
        queryEnergyEfficiencyDataDTO.setTimeType(EnergyEfficiencyTimeTypeEnum.MONTH.getType());
        PoemsFeignResponse<EnergyEfficiencyDataListVO> efficiencyDataListResponse =
                energyEfficiencyClient.queryProjectTargetDataList(JSON.toJSONString(queryEnergyEfficiencyDataDTO));

        // 解决返回结果，获取月能效值
        List<EnergyEfficiencyDataVO> dataList = analysisResponse(efficiencyDataListResponse);
        if (CollectionUtils.isEmpty(dataList)) {
            return null;
        }
        EnergyEfficiencyDataVO energyEfficiencyDataVO = dataList.get(0);
        if (energyEfficiencyDataVO == null) {
            return null;
        }
        return energyEfficiencyDataVO.getData();
    }

    private List<EnergyEfficiencyDataVO> analysisResponse(
            PoemsFeignResponse<EnergyEfficiencyDataListVO> efficiencyDataListResponse) {
        // 解决返回结果，获取月能效值
        if (efficiencyDataListResponse == null) {
            return null;
        }
        List<EnergyEfficiencyDataListVO> efficiencyDataList = efficiencyDataListResponse.getContent();
        if (CollectionUtils.isEmpty(efficiencyDataList)) {
            return null;
        }
        EnergyEfficiencyDataListVO energyEfficiencyDataListVO = efficiencyDataList.get(0);
        if (energyEfficiencyDataListVO == null) {
            return null;
        }
        return energyEfficiencyDataListVO.getList();
    }

    /**
     * 查询一个月内每日能效
     *
     * @param targetId  能效指标id
     * @param projectId 项目id
     * @param date      日期
     * @return 能效值
     * @author lixing
     * @version V1.0 2021/7/28 5:09 下午
     */
    public List<EnergyEfficiencyDataVO> getDailyEnergyEfficiency(String targetId, String projectId, Date date) {
        QueryEnergyEfficiencyDataDTO queryEnergyEfficiencyDataDTO = initQueryDTO(targetId, projectId, date);
        queryEnergyEfficiencyDataDTO.setTimeType(EnergyEfficiencyTimeTypeEnum.DAY.getType());
        PoemsFeignResponse<EnergyEfficiencyDataListVO> efficiencyDataListResponse =
                energyEfficiencyClient.queryProjectTargetDataList(JSON.toJSONString(queryEnergyEfficiencyDataDTO));
        return analysisResponse(efficiencyDataListResponse);
    }

    private QueryEnergyEfficiencyDataDTO initQueryDTO(String targetId, String projectId, Date date) {
        // 获取当月开始时间
        Date firstDayOfMonth = DateUtils.getFirstDayOfMonth(date);
        // 下月开始时间
        Date firstDayOfNextMonth = DateUtils.getMonthOff(firstDayOfMonth, 1);

        QueryEnergyEfficiencyDataDTO queryEnergyEfficiencyDataDTO = new QueryEnergyEfficiencyDataDTO();
        queryEnergyEfficiencyDataDTO.setTargetId(targetId);
        queryEnergyEfficiencyDataDTO.setProjectId(projectId);
        queryEnergyEfficiencyDataDTO.setTimeFrom(DateUtils.date2Str(firstDayOfMonth, DateUtils.SDF_SECOND));
        queryEnergyEfficiencyDataDTO.setTimeTo(DateUtils.date2Str(firstDayOfNextMonth, DateUtils.SDF_SECOND));

        return queryEnergyEfficiencyDataDTO;
    }
}
