package com.persagy.apm.report.indicator.factory;

import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.centermiddleware.constant.enums.TimeTypeEnum;
import com.persagy.apm.report.dependencies.centermiddleware.model.dto.QueryEnergyDataDTO;
import com.persagy.apm.report.dependencies.centermiddleware.service.ICenterMiddlewareWebService;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.indicator.codes.energyUsage.EnergyUsageContentCodes;
import com.persagy.apm.report.indicator.codes.energyUsage.EnergyUsageHeaderCodes;
import com.persagy.apm.report.indicator.model.entity.EnergyUsageDefine;
import com.persagy.apm.report.indicator.utils.IndicatorUtils;
import com.persagy.apm.report.paragraphs.service.impl.energyusage.ProjectEnergyUsageBuilder;
import com.persagy.apm.report.userconfig.function.model.Function;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * 能耗工厂
 *
 * @author lixing
 * @version V1.0 2021/7/20 3:55 下午
 **/
@Service
public class EnergyUsageFactory {
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    ICenterMiddlewareWebService centerMiddlewareWebService;
    @Autowired
    ProjectEnergyUsageBuilder projectEnergyUsageBuilder;

    /**
     * 获取指标单月用量
     *
     * @param energyUsageDefine 能耗用量指标对象
     * @param date              日期
     * @param projectId         项目id
     * @return 指标单月用量
     * @author lixing
     * @version V1.0 2021/7/20 4:00 下午
     */
    public Double getMonthlyUsageValue(EnergyUsageDefine energyUsageDefine, Date date, String projectId) {
        // 项目本地编码
        String projectLocalId = saasWebService.getProjectLocalId(projectId);
        TreeMap<Date, Double> itemEnergyDataMap = getItemEnergyDataMap(date, date, energyUsageDefine, projectLocalId);
        if (CollectionUtils.isEmpty(itemEnergyDataMap)) {
            return null;
        }
        return itemEnergyDataMap.get(date);
    }

    /**
     * 获取指标年度累计用量
     *
     * @param energyUsageDefine 能耗用量指标对象
     * @param date              日期
     * @param projectId         项目id
     * @return 指标年度累计用量
     * @author lixing
     * @version V1.0 2021/7/20 4:00 下午
     */
    public Double getAggregateUsageValue(EnergyUsageDefine energyUsageDefine, Date date, String projectId) {
        // 项目本地编码
        String projectLocalId = saasWebService.getProjectLocalId(projectId);
        // 获取统计时间当年的一月一日, 作为起始时间
        Date startDate = DateUtils.getFirstDayOfYear(date);
        TreeMap<Date, Double> itemEnergyDataMap = getItemEnergyDataMap(startDate, date, energyUsageDefine, projectLocalId);
        if (CollectionUtils.isEmpty(itemEnergyDataMap)) {
            return null;
        }

        Double result = null;
        // 将本年的用电量进行加和
        for (Double value : itemEnergyDataMap.values()) {
            if (value == null) {
                continue;
            }
            if (result == null) {
                result = 0d;
            }
            result = DataUtils.doubleSum(result, value);
        }
        return result;
    }

    /**
     * 获取条目能耗值map
     *
     * @param startDate         开始时间
     * @param endDate           结束时间
     * @param energyUsageDefine 能耗指标对象
     * @param projectLocalId    项目本地编码
     * @return 分项能耗信息{时间(Date)：数据(Double)}
     * @author lixing
     * @version V1.0 2021/5/30 11:39 上午
     */
    public TreeMap<Date, Double> getItemEnergyDataMap(
            Date startDate, Date endDate, EnergyUsageDefine energyUsageDefine, String projectLocalId) {
        QueryEnergyDataDTO queryEnergyDataDTO = new QueryEnergyDataDTO();
        queryEnergyDataDTO.setProjectId(projectLocalId);
        queryEnergyDataDTO.setBuildingId(projectLocalId);
        queryEnergyDataDTO.setModelCode(energyUsageDefine.getModelCode());
        queryEnergyDataDTO.setStartDate(DateUtils.date2Str(startDate, DateUtils.SDF_SECOND));
        // 结束时间往后延一个月，中间件时间查询为左闭右开
        queryEnergyDataDTO.setEndDate(DateUtils.date2Str(DateUtils.getMonthOff(endDate, 1), DateUtils.SDF_SECOND));
        queryEnergyDataDTO.setItemIdList(Lists.newArrayList(energyUsageDefine.getItemId()));
        queryEnergyDataDTO.setTimeType(TimeTypeEnum.MONTH.getType());
        Map<String, TreeMap<Date, Double>> itemEnergyDataMap = centerMiddlewareWebService.getItemEnergyDataMap(queryEnergyDataDTO);
        return itemEnergyDataMap.get(energyUsageDefine.getItemId());
    }

    /**
     * 获取能耗用量信息
     *
     * @param function    指标对象
     * @param define      能耗指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 能耗用量信息
     * @author lixing
     * @version V1.0 2021/7/22 6:39 下午
     */
    public ItemVO getEnergyUsageInfo(Function function, EnergyUsageDefine define, String projectId, Date reportMonth) {
        List<AttrValueVO> attrValues = new ArrayList<>();
        ItemVO itemVO = new ItemVO(function.getId(), function.getName(), attrValues);
        // 获取本月能耗
        AttrValueVO currentMonth = getCurrentMonth(define, projectId, reportMonth);
        attrValues.add(currentMonth);
        // 上月能耗
        AttrValueVO lastMonth = getLastMonth(define, projectId, reportMonth);
        attrValues.add(lastMonth);
        // 环比增量
        AttrValueVO chainGrowth = getChainGrowth(
                DataUtils.parseDouble(currentMonth.getValue()),
                DataUtils.parseDouble(lastMonth.getValue()), define.getUnit());
        attrValues.add(chainGrowth);
        // 环比增幅
        AttrValueVO chainRange = getChainRange(
                DataUtils.parseDouble(chainGrowth.getValue()), DataUtils.parseDouble(lastMonth.getValue()));
        attrValues.add(chainRange);
        // 去年同期用量
        AttrValueVO sameTime = getSameTime(define, projectId, reportMonth);
        attrValues.add(sameTime);
        // 同比增量
        AttrValueVO sameTimeGrowth = getSameTimeGrowth(
                DataUtils.parseDouble(currentMonth.getValue()), DataUtils.parseDouble(sameTime.getValue()), define.getUnit());
        attrValues.add(sameTimeGrowth);
        // 同比增幅
        AttrValueVO sameTimeRange = getSameTimeRange(
                DataUtils.parseDouble(sameTimeGrowth.getValue()), DataUtils.parseDouble(sameTime.getValue()));
        attrValues.add(sameTimeRange);
        // 当年累计用量
        AttrValueVO aggregate = getAggregate(define, projectId, reportMonth);
        attrValues.add(aggregate);
        // 去年同期累计用量
        AttrValueVO aggregateLastYear = getAggregateLastYear(define, projectId, reportMonth);
        attrValues.add(aggregateLastYear);
        // 累计同比增量
        AttrValueVO aggregateGrowth = getAggregateGrowth(
                DataUtils.parseDouble(aggregate.getValue()), DataUtils.parseDouble(aggregateLastYear.getValue()), define.getUnit());
        attrValues.add(aggregateGrowth);
        // 累计同比增幅
        AttrValueVO aggregateRange = getAggregateRange(
                DataUtils.parseDouble(aggregateGrowth.getValue()), DataUtils.parseDouble(aggregateLastYear.getValue()));
        attrValues.add(aggregateRange);
        /* 单方用量*/
        // 获取项目的总建筑面积
        Double area = projectEnergyUsageBuilder.getProjectBuildingArea(Lists.newArrayList(projectId));
        // 单位面积用量
        AttrValueVO perSqm = getPerSqm(DataUtils.parseDouble(currentMonth.getValue()), area,
                define.getUnit() + DataConstants.SQM);
        attrValues.add(perSqm);
        // 上月单位面积用量
        AttrValueVO perSqmLastMonth = getPerSqmLastMonth(DataUtils.parseDouble(lastMonth.getValue()), area,
                define.getUnit() + DataConstants.SQM);
        attrValues.add(perSqmLastMonth);
        // 去年同期单位面积用量
        AttrValueVO perSqmSameTime = getPerSqmSameTime(DataUtils.parseDouble(sameTime.getValue()), area,
                define.getUnit() + DataConstants.SQM);
        attrValues.add(perSqmSameTime);
        // 单位面积环比增量
        AttrValueVO perSqmChainGrowth = getPerSqmChainGrowth(
                DataUtils.parseDouble(perSqm.getValue()), DataUtils.parseDouble(perSqmLastMonth.getValue()),
                define.getUnit() + DataConstants.SQM);
        attrValues.add(perSqmChainGrowth);
        // 单位面积环比增幅
        AttrValueVO perSqmChainRange = getPerSqmChainRange(
                DataUtils.parseDouble(perSqmChainGrowth.getValue()), DataUtils.parseDouble(perSqmLastMonth.getValue()));
        attrValues.add(perSqmChainRange);
        // 单位面积同比增量
        AttrValueVO perSqmSameTimeGrowth = getPerSqmSameTimeGrowth(
                DataUtils.parseDouble(perSqm.getValue()), DataUtils.parseDouble(perSqmSameTime.getValue()),
                define.getUnit() + DataConstants.SQM);
        attrValues.add(perSqmSameTimeGrowth);
        // 单位面积同比增幅
        AttrValueVO perSqmSameTimeRange = getPerSqmSameTimeRange(
                DataUtils.parseDouble(perSqmSameTimeGrowth.getValue()), DataUtils.parseDouble(perSqmSameTime.getValue()));
        attrValues.add(perSqmSameTimeRange);
        return itemVO;
    }

    /**
     * 获取当月用量指标
     *
     * @param define      能耗指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月用量指标
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getCurrentMonth(EnergyUsageDefine define, String projectId, Date reportMonth) {
        Double currentMonth = getMonthlyUsageValue(define, reportMonth, projectId);
        String currentMonthStr = currentMonth == null ? null : String.valueOf(currentMonth);
        return new AttrValueVO(EnergyUsageContentCodes.currentMonth.name(),
                currentMonthStr, AttrValueTypeEnum.number_1, define.getUnit());
    }

    /**
     * 获取上月用量指标
     *
     * @param define      能耗指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月用量指标
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getLastMonth(EnergyUsageDefine define, String projectId, Date reportMonth) {
        Date firstDayOfLastMonth = DateUtils.getFirstDayOfLastMonth(reportMonth);
        Double lastMonth = getMonthlyUsageValue(define, firstDayOfLastMonth, projectId);
        String lastMonthStr = lastMonth == null ? null : String.valueOf(lastMonth);
        return new AttrValueVO(EnergyUsageContentCodes.lastMonth.name(),
                lastMonthStr, AttrValueTypeEnum.number_1, define.getUnit());
    }

    /**
     * 获取环比增量  当月-上月
     *
     * @param currentMonth 当月用量
     * @param lastMonth    上月用量
     * @param unit         单位
     * @return 环比增量
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getChainGrowth(Double currentMonth, Double lastMonth, String unit) {
        AttrValueVO growth = IndicatorUtils.getGrowth(EnergyUsageContentCodes.chainGrowth.name(),
                currentMonth, lastMonth, AttrValueTypeEnum.number_1);
        growth.setUnit(unit);
        return growth;
    }

    /**
     * 获取环比增幅  环比增量/上月用量
     *
     * @param chainGrowth 环比增量
     * @param lastMonth   上月用量
     * @return 环比增幅
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getChainRange(Double chainGrowth, Double lastMonth) {
        return IndicatorUtils.getRange(EnergyUsageContentCodes.chainRange.name(),
                chainGrowth, lastMonth, AttrValueTypeEnum.percent_1);
    }

    /**
     * 获取去年同期用量
     *
     * @param define      能耗指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月用量指标
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getSameTime(EnergyUsageDefine define, String projectId, Date reportMonth) {
        Date sameTimeLastYear = DateUtils.getSameMonthFirstDayOfLastYear(reportMonth);
        Double sameTimeValue = getMonthlyUsageValue(define, sameTimeLastYear, projectId);
        String sameTimeValueStr = sameTimeValue == null ? null : String.valueOf(sameTimeValue);
        return new AttrValueVO(EnergyUsageContentCodes.sameTime.name(),
                sameTimeValueStr, AttrValueTypeEnum.number_1, define.getUnit());
    }

    /**
     * 获取同比增量  当月-去年同期
     *
     * @param currentMonth     当月用量
     * @param sametimeLastYear 去年同期用量
     * @param unit             单位
     * @return 同比增量
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getSameTimeGrowth(Double currentMonth, Double sametimeLastYear, String unit) {
        AttrValueVO growth = IndicatorUtils.getGrowth(EnergyUsageContentCodes.sametimeGrowth.name(),
                currentMonth, sametimeLastYear, AttrValueTypeEnum.number_1);
        growth.setUnit(unit);
        return growth;
    }

    /**
     * 获取同比增幅  同比增量/去年同期用量
     *
     * @param sametimeGrowth   同比增量
     * @param sametimeLastYear 去年同期用量
     * @return 同比增幅
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getSameTimeRange(Double sametimeGrowth, Double sametimeLastYear) {
        return IndicatorUtils.getRange(EnergyUsageContentCodes.sametimeRange.name(),
                sametimeGrowth, sametimeLastYear, AttrValueTypeEnum.percent_1);
    }

    /**
     * 获取指标本年累计用量
     *
     * @param define      能耗指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 本年累计用量
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getAggregate(EnergyUsageDefine define, String projectId, Date reportMonth) {
        Double aggregateUsageValue = getAggregateUsageValue(define, reportMonth, projectId);
        String aggregateUsageValueStr = aggregateUsageValue == null ? null : String.valueOf(aggregateUsageValue);
        return new AttrValueVO(EnergyUsageContentCodes.aggregate.name(),
                aggregateUsageValueStr, AttrValueTypeEnum.number_1, define.getUnit());
    }

    /**
     * 获取指标去年同期累计用量
     *
     * @param define      能耗指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月用量指标
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getAggregateLastYear(EnergyUsageDefine define, String projectId, Date reportMonth) {
        Date sameTimeLastYear = DateUtils.getSameMonthFirstDayOfLastYear(reportMonth);
        Double aggregateLastYearValue = getAggregateUsageValue(define, sameTimeLastYear, projectId);
        String aggregateLastYearValueStr = aggregateLastYearValue == null ?
                null : String.valueOf(aggregateLastYearValue);
        return new AttrValueVO(EnergyUsageContentCodes.aggregateLastYear.name(),
                aggregateLastYearValueStr, AttrValueTypeEnum.number_1, define.getUnit());
    }

    /**
     * 获取累计同比增量  当年累计-去年同期累计
     *
     * @param aggregate         当年累计用量
     * @param aggregateLastYear 去年同期累计用量
     * @param unit              单位
     * @return 累计同比增量
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getAggregateGrowth(Double aggregate, Double aggregateLastYear, String unit) {
        AttrValueVO growth = IndicatorUtils.getGrowth(EnergyUsageContentCodes.aggregateGrowth.name(),
                aggregate, aggregateLastYear, AttrValueTypeEnum.number_1);
        growth.setUnit(unit);
        return growth;
    }

    /**
     * 获取累计同比增幅  同比增量/去年同期用量
     *
     * @param aggregateGrowth   累计同比增量
     * @param aggregateLastYear 去年同期累计用量
     * @return 累计同比增幅
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getAggregateRange(Double aggregateGrowth, Double aggregateLastYear) {
        return IndicatorUtils.getRange(EnergyUsageContentCodes.aggregateRange.name(),
                aggregateGrowth, aggregateLastYear, AttrValueTypeEnum.percent_1);
    }

    /**
     * 获取指标当月单方用量
     *
     * @param currentMonth 指标当月用量
     * @param area         面积
     * @param unit         单位
     * @return 指标当月单方用量
     * @author lixing
     * @version V1.0 2021/7/23 6:17 下午
     */
    public AttrValueVO getPerSqm(Double currentMonth, Double area, String unit) {
        Double perSqmValue = IndicatorUtils.getPerSqmValue(currentMonth, area);
        String perSqmValueStr = perSqmValue == null ? null : String.valueOf(perSqmValue);
        return new AttrValueVO(EnergyUsageContentCodes.perSqm.name(),
                perSqmValueStr, AttrValueTypeEnum.number_1, unit);
    }

    /**
     * 获取指标上月单方用量
     *
     * @param lastMonth 指标上月用量
     * @param area      面积
     * @param unit      单位
     * @return 指标上月单方用量
     * @author lixing
     * @version V1.0 2021/7/23 6:17 下午
     */
    public AttrValueVO getPerSqmLastMonth(Double lastMonth, Double area, String unit) {
        Double perSqmValue = IndicatorUtils.getPerSqmValue(lastMonth, area);
        String perSqmValueStr = perSqmValue == null ? null : String.valueOf(perSqmValue);
        return new AttrValueVO(EnergyUsageContentCodes.perSqmLastMonth.name(),
                perSqmValueStr, AttrValueTypeEnum.number_1, unit);
    }

    /**
     * 获取指标去年同期单方用量
     *
     * @param sametime 指标去年同期用量
     * @param area     面积
     * @param unit     单位
     * @return 指标去年同期单方用量
     * @author lixing
     * @version V1.0 2021/7/23 6:17 下午
     */
    public AttrValueVO getPerSqmSameTime(Double sametime, Double area, String unit) {
        Double perSqmValue = IndicatorUtils.getPerSqmValue(sametime, area);
        String perSqmValueStr = perSqmValue == null ? null : String.valueOf(perSqmValue);
        return new AttrValueVO(EnergyUsageContentCodes.perSqmSameTime.name(),
                perSqmValueStr, AttrValueTypeEnum.number_1, unit);
    }

    /**
     * 获取指标单位面积环比增量
     *
     * @param perSqm          指标当月单方用量
     * @param perSqmLastMonth 指标上月单方用量
     * @param unit            单位
     * @return 指标单位面积环比增量
     * @author lixing
     * @version V1.0 2021/7/23 6:28 下午
     */
    public AttrValueVO getPerSqmChainGrowth(Double perSqm, Double perSqmLastMonth, String unit) {
        AttrValueVO growth = IndicatorUtils.getGrowth(EnergyUsageContentCodes.perSqmChainGrowth.name(),
                perSqm, perSqmLastMonth, AttrValueTypeEnum.number_1);
        growth.setUnit(unit);
        return growth;
    }

    /**
     * 获取指标单位面积环比增幅
     *
     * @param chainGrowthPerSqm 指标单方用量环比增量
     * @param perSqmLastMonth   指标上月单方用量
     * @return 指标单位面积环比增幅
     * @author lixing
     * @version V1.0 2021/7/23 6:28 下午
     */
    public AttrValueVO getPerSqmChainRange(Double chainGrowthPerSqm, Double perSqmLastMonth) {
        return IndicatorUtils.getRange(EnergyUsageContentCodes.perSqmChainRange.name(),
                chainGrowthPerSqm, perSqmLastMonth, AttrValueTypeEnum.number_1);
    }

    /**
     * 获取指标单位面积同比增量
     *
     * @param perSqm         指标当月单方用量
     * @param perSqmSameTime 指标去年同期单方用量
     * @param unit           单位
     * @return 指标单位面积同比增量
     * @author lixing
     * @version V1.0 2021/7/23 6:28 下午
     */
    public AttrValueVO getPerSqmSameTimeGrowth(Double perSqm, Double perSqmSameTime, String unit) {
        AttrValueVO growth = IndicatorUtils.getGrowth(EnergyUsageContentCodes.perSqmSameTimeGrowth.name(),
                perSqm, perSqmSameTime, AttrValueTypeEnum.number_1);
        growth.setUnit(unit);
        return growth;
    }

    /**
     * 获取指标单位面积同比增幅
     *
     * @param sametimeGrowthPerSqm 指标单方用量同比增量
     * @param perSqmSametime       指标去年同期单方用量
     * @return 指标单位面积同比增幅
     * @author lixing
     * @version V1.0 2021/7/23 6:28 下午
     */
    public AttrValueVO getPerSqmSameTimeRange(Double sametimeGrowthPerSqm, Double perSqmSametime) {
        return IndicatorUtils.getRange(EnergyUsageContentCodes.perSqmSameTimeRange.name(),
                sametimeGrowthPerSqm, perSqmSametime, AttrValueTypeEnum.number_1);
    }

    /**
     * 获取同等碳排放量
     *
     * @param usage 能耗量
     * @return 同等碳排放量
     * @author lixing
     * @version V1.0 2021/7/26 2:23 下午
     */
    public AttrValueVO getTce(Double usage) {
        Double slewRate = 2500d;
        // 这里使用通用的getRange方法来计算用量除以转换率的值，并封装成对象
        AttrValueVO range = IndicatorUtils.getRange(EnergyUsageHeaderCodes.tce.name(),
                usage, slewRate, AttrValueTypeEnum.number_1);
        // getRange方法默认的单位为"%"，这里将单位置空
        range.setUnit("");
        return range;
    }
}
