package com.persagy.apm.report.indicator.factory;

import com.alibaba.excel.util.CollectionUtils;
import com.google.gson.Gson;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.dependencies.dmpalarm.enums.EnumAlarmTreatState;
import com.persagy.apm.report.dependencies.dmpalarm.model.AlarmRecord;
import com.persagy.apm.report.dependencies.dmpalarm.service.AlarmServiceImpl;
import com.persagy.apm.report.dependencies.environment.service.IEnvironmentWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.indicator.codes.env.ProjectEnvContentCodes;
import com.persagy.apm.report.indicator.model.entity.EnvComplianceRateQualifiedStandard;
import com.persagy.apm.report.indicator.model.entity.EnvDefine;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.statisticsitems.enums.QualifiedEnum;
import com.persagy.apm.report.userconfig.statisticsitems.enums.StatisticCodeEnum;
import com.persagy.apm.report.userconfig.statisticsitems.model.StatisticItem;
import com.persagy.apm.report.userconfig.statisticsitems.service.IStatisticItemService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 环境指标工厂
 *
 * @author lixing
 * @version V1.0 2021/8/4 3:40 下午
 **/
@Service
public class EnvFactory {
    @Autowired
    IEnvironmentWebService environmentWebService;
    @Autowired
    AlarmServiceImpl alarmService;
    @Autowired
    IStatisticItemService statisticItemService;

    private Map<String, String> getAlarmTypeMap() {
        Map<String, String> result = new HashMap<>();
        // 湿度过低
        result.put("HJBJ_04", "T000033");
        // 湿度过高
        result.put("HJBJ_03", "T000032");
        // 温度过低
        result.put("HJBJ_02", "T000031");
        // 温度过高
        result.put("HJBJ_01", "T000030");
        // PM2.5过高
        result.put("HJBJ_06", "T000034");
        // CO2过高
        result.put("HJBJ_05", "T000035");
        // CO过高
        result.put("HJBJ_09", "T000036");
        // 甲醛过高
        result.put("HJBJ_07", "T000037");
        // PM10过高
        result.put("HJBJ_11", "T000038");
        // TVOC过高
        result.put("HJBJ_13", "T000039");
        // O3过高
        result.put("HJBJ_10", "T000238");
        // Rn过高
        result.put("HJBJ_12", "T000239");
        return result;
    }

    /**
     * 根据环境健康的报警类型获取数据中台的报警类型
     *
     * @param envAlarmType 环境健康的报警类型
     * @return 数据中台的报警类型
     * @author lixing
     * @version V1.0 2021/8/16 12:02 下午
     */
    public String getDmpAlarmType(String envAlarmType) {
        if (StringUtils.isBlank(envAlarmType)) {
            return null;
        }
        Map<String, String> alarmTypeMap = getAlarmTypeMap();
        return alarmTypeMap.get(envAlarmType);
    }

    /**
     * 根据环境健康的报警类型获取数据中台的报警类型
     *
     * @param envAlarmTypeList 环境健康的报警类型
     * @return 数据中台的报警类型
     * @author lixing
     * @version V1.0 2021/8/16 12:02 下午
     */
    public List<String> getDmpAlarmType(List<String> envAlarmTypeList) {
        if (CollectionUtils.isEmpty(envAlarmTypeList)) {
            return null;
        }
        List<String> result = new ArrayList<>();
        for (String envAlarmType : envAlarmTypeList) {
            String dmpAlarmType = getDmpAlarmType(envAlarmType);
            if (StringUtils.isNotBlank(dmpAlarmType)) {
                result.add(dmpAlarmType);
            }
        }
        return result;

    }

    /**
     * 获取达标率
     *
     * @param envDefine 环境指标定义
     * @param projectId 项目id
     * @param date      达标率月份
     * @return 达标率
     * @author lixing
     * @version V1.0 2021/8/4 3:42 下午
     */
    public AttrValueVO getComplianceRate(EnvDefine envDefine, String projectId, Date date) {
        Double envComplianceRate = environmentWebService.getEnvComplianceRate(projectId, date, envDefine);
        String envComplianceRateStr = envComplianceRate == null ? null : String.valueOf(envComplianceRate);
        return new AttrValueVO(ProjectEnvContentCodes.complianceRate.name(),
                envComplianceRateStr, AttrValueTypeEnum.number_2, DataConstants.PERCENT);
    }

    /**
     * 获取指标相关的报警
     *
     * @param envDefine 环境指标定义
     * @param projectId 项目id
     * @param month     月份
     * @return 指标相关的报警
     * @author lixing
     * @version V1.0 2021/8/4 3:44 下午
     */
    public List<AlarmRecord> queryRelAlarms(EnvDefine envDefine, String projectId, Date month) {
        List<String> itemCodes = environmentWebService.queryEnvAlarmItems(envDefine);
        List<String> dmpAlarmItemCodes = getDmpAlarmType(itemCodes);
        return alarmService.queryAlarmByProjectIdAndItemCodesAndMonth(projectId, dmpAlarmItemCodes, month);
    }

    /**
     * 获取环境指标报警次数
     *
     * @param relAlarmRecords 环境指标关联的报警
     * @return 环境指标报警次数
     * @author lixing
     * @version V1.0 2021/8/4 4:14 下午
     */
    public AttrValueVO getAlarmCount(List<AlarmRecord> relAlarmRecords) {
        String alarmCount = null;
        if (!CollectionUtils.isEmpty(relAlarmRecords)) {
            alarmCount = String.valueOf(relAlarmRecords.size());
        }
        return new AttrValueVO(ProjectEnvContentCodes.alarmCount.name(),
                alarmCount, AttrValueTypeEnum.number);
    }

    /**
     * 获取环境指标已处理报警次数
     *
     * @param relAlarmRecords 环境指标关联的报警
     * @return 环境指标已处理报警次数
     * @author lixing
     * @version V1.0 2021/8/4 4:14 下午
     */
    public AttrValueVO getHandledAlarmCount(List<AlarmRecord> relAlarmRecords) {
        String handledAlarmCount = null;
        if (!CollectionUtils.isEmpty(relAlarmRecords)) {
            long count = relAlarmRecords.stream().filter(
                    alarmRecord -> EnumAlarmTreatState.DONE.getType().equals(alarmRecord.getTreatState())).count();
            handledAlarmCount = String.valueOf(count);
        }
        return new AttrValueVO(ProjectEnvContentCodes.handledAlarmCount.name(),
                handledAlarmCount, AttrValueTypeEnum.number);
    }

    /**
     * 获取环境指标报警处理率
     *
     * @param handledCount 已处理报警数
     * @param totalCount   总报警数
     * @return 报警处理率
     * @author lixing
     * @version V1.0 2021/8/4 4:14 下午
     */
    public AttrValueVO getHandleProportion(Double handledCount, Double totalCount) {
        if (handledCount == null || totalCount == null) {
            return new AttrValueVO(ProjectEnvContentCodes.handleProportion.name(),
                    null, AttrValueTypeEnum.percent_2, DataConstants.PERCENT);
        }
        Double proportion = DataUtils.doubleDivide(handledCount, totalCount);
        return new AttrValueVO(ProjectEnvContentCodes.handleProportion.name(),
                String.valueOf(proportion), AttrValueTypeEnum.percent_2, DataConstants.PERCENT);
    }

    /**
     * 获取合格标准
     *
     * @param function 指标对象
     * @return 合格标准
     * @author lixing
     * @version V1.0 2021/8/2 6:49 下午
     */
    public String getQualifiedStandardValue(Function function) {
        String standardEnvComplianceRate = null;
        String qualifiedStandardStr = function.getQualifiedStandard();
        if (StringUtils.isNotBlank(qualifiedStandardStr)) {
            Gson gson = new Gson();
            EnvComplianceRateQualifiedStandard envComplianceRateQualifiedStandard = gson.fromJson(
                    qualifiedStandardStr, EnvComplianceRateQualifiedStandard.class);
            standardEnvComplianceRate = envComplianceRateQualifiedStandard.getStandardEnvComplianceRate();
        }

        if (StringUtils.isBlank(standardEnvComplianceRate)) {
            // 获取达标率标准判断条件
            StatisticItem statisticItem = statisticItemService.getItemByCode(
                    StatisticCodeEnum.ENV_QUALIFIED_PROPORTION.name());
            if (statisticItem != null) {
                standardEnvComplianceRate = statisticItem.getStandardValue();
            }
        }
        return standardEnvComplianceRate;
    }

    /**
     * 获取合格标准
     *
     * @param function 指标对象
     * @return 合格标准
     * @author lixing
     * @version V1.0 2021/8/2 6:49 下午
     */
    public AttrValueVO getQualifiedStandard(Function function) {
        return new AttrValueVO(ProjectEnvContentCodes.complianceRateQualifiedStandard.name(),
                getQualifiedStandardValue(function), AttrValueTypeEnum.percent_2, DataConstants.PERCENT);
    }

    /**
     * 获取是否合格
     *
     * @param currentMonth  本月达标率
     * @param standardValue 达标率标准值
     * @return 是否合格
     * @author lixing
     * @version V1.0 2021/8/2 6:49 下午
     */
    public AttrValueVO getIsQualified(Double currentMonth, Double standardValue) {
        if (currentMonth == null || standardValue == null) {
            return new AttrValueVO(ProjectEnvContentCodes.complianceRateIsQualified.name(),
                    null, AttrValueTypeEnum.str);
        }
        // 获取达标率标准判断条件
        StatisticItem statisticItem = statisticItemService.getItemByCode(StatisticCodeEnum.ENV_QUALIFIED_PROPORTION.name());
        Double qualifyResult = DataUtils.getQualifyResult(standardValue, statisticItem.getQualifyFormula(), currentMonth);

        QualifiedEnum qualifiedEnum = QualifiedEnum.getByType(qualifyResult);
        if (qualifiedEnum == null) {
            return new AttrValueVO(ProjectEnvContentCodes.complianceRateIsQualified.name(),
                    null, AttrValueTypeEnum.str);
        }
        return new AttrValueVO(ProjectEnvContentCodes.complianceRateIsQualified.name(),
                qualifiedEnum.getDesc(), AttrValueTypeEnum.str);
    }

    /**
     * 获取环境指标信息
     *
     * @param function    环境指标对象
     * @param envDefine   环境指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 环境指标信息
     * @author lixing
     * @version V1.0 2021/8/4 7:33 下午
     */
    public ItemVO getEnvInfo(Function function, EnvDefine envDefine, String projectId, Date reportMonth) {
        List<AttrValueVO> attrValues = new ArrayList<>();
        ItemVO itemVO = new ItemVO(function.getId(), function.getName(), attrValues);

        // 查询当月指标关联的报警
        List<AlarmRecord> alarmRecords = queryRelAlarms(envDefine, projectId, reportMonth);
        // 达标率
        AttrValueVO complianceRate = getComplianceRate(envDefine, projectId, reportMonth);
        attrValues.add(complianceRate);
        // 报警次数
        AttrValueVO alarmCount = getAlarmCount(alarmRecords);
        attrValues.add(alarmCount);
        // 已处理报警数
        AttrValueVO handledAlarmCount = getHandledAlarmCount(alarmRecords);
        attrValues.add(handledAlarmCount);
        // 报警处理率
        AttrValueVO handleProportion = getHandleProportion(
                DataUtils.parseDouble(handledAlarmCount.getValue()), DataUtils.parseDouble(alarmCount.getValue()));
        attrValues.add(handleProportion);
        // 合格标准
        AttrValueVO qualifiedStandard = getQualifiedStandard(function);
        attrValues.add(qualifiedStandard);
        // 是否合格
        AttrValueVO isQualified = getIsQualified(
                DataUtils.parseDouble(complianceRate.getValue()), DataUtils.parseDouble(qualifiedStandard.getValue()));
        attrValues.add(isQualified);

        return itemVO;
    }
}
