package com.persagy.apm.report.indicator.factory;

import com.google.gson.Gson;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.iotdataanalysis.service.IDataAnalysisWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.indicator.codes.iot.ProjectIotContentCodes;
import com.persagy.apm.report.indicator.model.entity.IotDefine;
import com.persagy.apm.report.indicator.model.entity.IotQualifiedStandard;
import com.persagy.apm.report.indicator.model.vo.GraphItemVO;
import com.persagy.apm.report.indicator.model.vo.GraphVO;
import com.persagy.apm.report.paragraphs.constants.StandardNames;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.statisticsitems.enums.QualifiedEnum;
import com.persagy.apm.report.userconfig.statisticsitems.model.StatisticItem;
import com.persagy.apm.report.userconfig.statisticsitems.service.IStatisticItemService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 数据质量指标工厂
 *
 * @author lixing
 * @version V1.0 2021/8/5 5:48 下午
 **/
@Service
public class IotFactory {
    @Autowired
    IDataAnalysisWebService iDataAnalysisWebService;
    @Autowired
    IStatisticItemService statisticItemService;

    /**
     * 本月取值
     *
     * @param iotDefine 数据质量指标定义
     * @param projectId 项目id
     * @param date      取值月份
     * @return 本月取值
     * @author lixing
     * @version V1.0 2021/8/5 5:57 下午
     */
    public Double getCurrentMonthValue(IotDefine iotDefine, String projectId, Date date) {
        return iDataAnalysisWebService.getTargetValue(projectId, date, iotDefine.getTargetCode());
    }

    /**
     * 本月取值
     *
     * @param iotDefine 数据质量指标定义
     * @param projectId 项目id
     * @param date      取值月份
     * @return 本月取值
     * @author lixing
     * @version V1.0 2021/8/5 5:57 下午
     */
    public AttrValueVO getCurrentMonth(IotDefine iotDefine, String projectId, Date date) {
        Double currentMonthValue = getCurrentMonthValue(iotDefine, projectId, date);
        if (currentMonthValue != null) {
            // 达标率返回值百分比的分子，这里除以100，返回原始数据
            currentMonthValue = DataUtils.doubleDivide(currentMonthValue, 100d);
        }
        String currentMonthStr = currentMonthValue == null ? null : String.valueOf(currentMonthValue);
        return new AttrValueVO(ProjectIotContentCodes.currentMonth.name(),
                currentMonthStr,
                AttrValueTypeEnum.percent_2, DataConstants.PERCENT);
    }

    /**
     * 当年每月的取值
     *
     * @param function  指标对象
     * @param projectId 项目id
     * @param date      取值月份
     * @return 当年每月的取值
     * @author lixing
     * @version V1.0 2021/8/5 5:57 下午
     */
    public GraphVO getYearGraph(Function function, String projectId, Date date) {
        Gson gson = new Gson();
        IotDefine iotDefine = gson.fromJson(function.getDefine(), IotDefine.class);
        // 获取当年每个月份
        Date firstDayOfYear = DateUtils.getFirstDayOfYear(date);
        Date decemberFirstDayOfYear = DateUtils.getDecemberFirstDayOfYear(date);
        List<Date> firstDayOfEveryMonth = DateUtils.getFirstDayOfEveryMonth(firstDayOfYear, decemberFirstDayOfYear);

        List<GraphItemVO> data = new ArrayList<>();
        GraphVO graphVO = new GraphVO(StandardNames.STANDARD_RANGE, getQualifiedStandardValue(function),
                AttrValueTypeEnum.percent, data);

        for (Date tmpDate : firstDayOfEveryMonth) {
            Double currentMonthValue = null;
            if (tmpDate.before(date)) {
                currentMonthValue = getCurrentMonthValue(iotDefine, projectId, tmpDate);
            }
            GraphItemVO graphItemVO = new GraphItemVO(
                    DateUtils.date2Str(tmpDate, DateUtils.SDF_SPACE_SECOND),
                    currentMonthValue, AttrValueTypeEnum.number_2, DataConstants.PERCENT);
            data.add(graphItemVO);
        }

        return graphVO;
    }

    /**
     * 当年每月的取值
     *
     * @param function  指标对象
     * @param projectId 项目id
     * @param date      取值月份
     * @return 当年每月的取值
     * @author lixing
     * @version V1.0 2021/8/5 5:57 下午
     */
    public AttrValueVO getMonthOfYear(Function function, String projectId, Date date) {
        GraphVO yearGraph = getYearGraph(function, projectId, date);
        Gson gson = new Gson();
        return new AttrValueVO(ProjectIotContentCodes.monthOfYear.name(),
                gson.toJson(yearGraph), AttrValueTypeEnum.graph);
    }

    /**
     * 获取合格标准
     *
     * @param function 指标对象
     * @return 合格标准
     * @author lixing
     * @version V1.0 2021/8/2 6:49 下午
     */
    public String getQualifiedStandardValue(Function function) {
        String standardRange = null;
        Gson gson = new Gson();
        String qualifiedStandardStr = function.getQualifiedStandard();
        if (StringUtils.isNotBlank(qualifiedStandardStr)) {
            IotQualifiedStandard iotQualifiedStandard = gson.fromJson(
                    qualifiedStandardStr, IotQualifiedStandard.class);
            standardRange = iotQualifiedStandard.getStandardRange();
        }

        if (StringUtils.isBlank(standardRange)) {
            IotDefine iotDefine = gson.fromJson(function.getDefine(), IotDefine.class);
            // 获取通用统计条目配置
            StatisticItem statisticItem = statisticItemService.getItemByCode(
                    iotDefine.getTargetCode());
            if (statisticItem != null) {
                standardRange = statisticItem.getStandardValue();
            }
        }
        return standardRange;
    }

    /**
     * 获取合格标准
     *
     * @param function 指标对象
     * @return 合格标准
     * @author lixing
     * @version V1.0 2021/8/2 6:49 下午
     */
    public AttrValueVO getQualifiedStandard(Function function) {
        return new AttrValueVO(ProjectIotContentCodes.qualifiedRange.name(),
                getQualifiedStandardValue(function), AttrValueTypeEnum.percent_2, DataConstants.PERCENT);
    }

    /**
     * 获取是否合格
     *
     * @param currentMonth  本月达标率
     * @param standardValue 达标率标准值
     * @return 是否合格
     * @author lixing
     * @version V1.0 2021/8/2 6:49 下午
     */
    public AttrValueVO getIsQualified(Double currentMonth, Double standardValue, String targetCode) {
        if (currentMonth == null || standardValue == null) {
            return new AttrValueVO(ProjectIotContentCodes.isQualified.name(),
                    null, AttrValueTypeEnum.str);
        }
        // 获取达标率标准判断条件
        StatisticItem statisticItem = statisticItemService.getItemByCode(targetCode);
        Double qualifyResult = DataUtils.getQualifyResult(standardValue, statisticItem.getQualifyFormula(), currentMonth);

        QualifiedEnum qualifiedEnum = QualifiedEnum.getByType(qualifyResult);
        if (qualifiedEnum == null) {
            return new AttrValueVO(ProjectIotContentCodes.isQualified.name(),
                    null, AttrValueTypeEnum.str);
        }
        return new AttrValueVO(ProjectIotContentCodes.isQualified.name(),
                qualifiedEnum.getDesc(), AttrValueTypeEnum.str);
    }

    /**
     * 获取数据质量指标信息
     *
     * @param function    数据质量指标对象
     * @param iotDefine   数据质量指标定义
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 数据质量指标信息
     * @author lixing
     * @version V1.0 2021/8/4 7:33 下午
     */
    public ItemVO getIotInfo(Function function, IotDefine iotDefine, String projectId, Date reportMonth) {
        List<AttrValueVO> attrValues = new ArrayList<>();
        ItemVO itemVO = new ItemVO(function.getId(), function.getName(), attrValues);

        // 本月取值
        AttrValueVO currentMonth = getCurrentMonth(iotDefine, projectId, reportMonth);
        attrValues.add(currentMonth);
        // 每月取值
        AttrValueVO monthOfYear = getMonthOfYear(function, projectId, reportMonth);
        attrValues.add(monthOfYear);
        // 合格标准
        AttrValueVO qualifiedStandard = getQualifiedStandard(function);
        attrValues.add(qualifiedStandard);
        // 是否合格
        AttrValueVO isQualified = getIsQualified(DataUtils.parseDouble(currentMonth.getValue()),
                DataUtils.parseDouble(qualifiedStandard.getValue()), iotDefine.getTargetCode());
        attrValues.add(isQualified);

        return itemVO;
    }
}
