package com.persagy.apm.report.outline.service.builder.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.reporttype.model.ReportType;
import com.persagy.apm.report.userconfig.reporttype.service.IReportTypeService;
import com.persagy.apm.report.userconfig.reporttype.util.ReportTypeUtil;
import com.persagy.apm.report.detail.paragraphvalue.service.IReportParagraphValueService;
import com.persagy.apm.report.outline.constants.enums.ReportStateEnum;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import com.persagy.apm.report.outline.service.builder.IReportGenerator;
import com.persagy.apm.report.paragraphs.service.IParagraphBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 商业项目报告建造者
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class ReportGenerator implements IReportGenerator {
    @Autowired
    private IReportTypeService reportTypeService;
    @Autowired
    private IReportOutlineService reportOutlineService;
    @Autowired
    private IReportParagraphValueService paragraphValueService;
    @Autowired
    private IParagraphService paragraphService;
//    @Autowired
//    private ReportGenerator reportGenerator;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void generateReport(ReportOutline reportOutline) {
        // 删除之前生成的段落
        deleteReportParagraphs(reportOutline);
        // 生成新的报告详情
//        boolean generateSuccess = reportGenerator.generateParagraphs(reportOutline);
        boolean generateSuccess = ((ReportGenerator)AopContext.currentProxy()).generateParagraphs(reportOutline);

        // 更新报告概要的状态
        if (generateSuccess) {
            // 更新报告生成状态为已生成
            updateReportOutlineInfoWhenSuccess(reportOutline);
        } else {
            // 生成失败，回滚上述生成报告操作，更新报告状态为生成失败
            updateReportOutlineInfoWhenFailure(reportOutline);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.NESTED)
    public boolean generateParagraphs(ReportOutline reportOutline) {
        String reportTypeId = reportOutline.getReportTypeId();
        ReportType reportType = reportTypeService.queryReportTypeDetail(reportTypeId);
        // 获取报告类型关联的段落
        List<String> paragraphIdList = ReportTypeUtil.getParagraphs(reportType);
        if (CollectionUtils.isEmpty(paragraphIdList)) {
            log.error("未获取到报告关联的段落信息，请联系管理员");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
        // 找到对应的段落建造者获取段落的取值
        for (String paragraphId : paragraphIdList) {
            Paragraph paragraph = paragraphService.queryParagraphDetail(paragraphId);
            String modelCode = paragraphService.getModelCode(paragraph);
            // 根据段落模板编码找到对应的段落建造者获取段落的取值
            IParagraphBuilder paragraphBuilder = SpringUtil.getBean(modelCode + "Builder");
            if (paragraphBuilder == null) {
                log.error("目前不能生成模板编码为：{}的段落", modelCode);
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
            paragraphBuilder.saveParagraphValue(paragraph, reportOutline);
        }
        return true;
    }

    @Override
    public void updateReportOutlineInfoWhenSuccess(ReportOutline reportOutline) {
        reportOutline.setStatus(ReportStateEnum.GENERATED.getType());
        reportOutlineService.updateReportOutline(reportOutline);
    }

    @Override
    public void updateReportOutlineInfoWhenFailure(ReportOutline reportOutline) {
        reportOutline.setStatus(ReportStateEnum.BUILD_FAILED.getType());
        reportOutlineService.updateReportOutline(reportOutline);
    }

    @Override
    public void deleteReportParagraphs(ReportOutline reportOutline) {
        // 删除这些段落之前的取值信息
        paragraphValueService.deleteByReport(reportOutline.getId());
    }
}
