package com.persagy.apm.report.paragraphs.service.impl;

import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrDefineVO;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.dto.AddReportParagraphValueDTO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.detail.paragraphvalue.service.IReportParagraphValueService;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.paragraphs.service.IParagraphBuilder;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.paragraphcustom.model.ParagraphCustom;
import com.persagy.apm.report.userconfig.paragraphcustom.service.IParagraphCustomService;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 段落建造者实现类
 *
 * @author lixing
 * @version V1.0 2021/7/20 9:42 上午
 **/
@Service
public abstract class ParagraphBuilder implements IParagraphBuilder {
    @Autowired
    IParagraphCustomService paragraphCustomService;
    @Autowired
    IReportParagraphValueService paragraphValueService;
    @Autowired
    IParagraphService paragraphService;
    @Autowired
    IFunctionService functionService;


    @Override
    public void saveParagraphValue(Paragraph paragraph, ReportOutline reportOutline) {
        // 保存段落取值信息
        AddReportParagraphValueDTO addReportParagraphValueDTO = new AddReportParagraphValueDTO();
        addReportParagraphValueDTO.setParagraphId(paragraph.getId());
        addReportParagraphValueDTO.setReportOutlineId(reportOutline.getId());

        // 计算段落内容部分
        ParagraphContentVO paragraphContent = getParagraphContent(paragraph, reportOutline);
        addReportParagraphValueDTO.setContent(paragraphContent);
        // 头部信息从段落内容中提取
        addReportParagraphValueDTO.setHeader(getHeader(paragraphContent, reportOutline));
        // 从段落定制中获取默认的补充说明
        addReportParagraphValueDTO.setSupplement(getSupplementValue(paragraph));

        paragraphValueService.createReportParagraphValue(addReportParagraphValueDTO);
    }

    /**
     * 计算段落内容部分
     *
     * @param paragraph     段落对象
     * @param reportOutline 报告概要对象
     * @return 段落内容部分
     * @author lixing
     * @version V1.0 2021/8/2 9:43 上午
     */
    protected abstract ParagraphContentVO getParagraphContent(Paragraph paragraph, ReportOutline reportOutline);

    /**
     * 获取段落的补充信息
     *
     * @param paragraph 段落对象
     * @return 段落的补充信息
     * @author lixing
     * @version V1.0 2021/7/22 11:16 上午
     */
    private List<AttrValueVO> getSupplementValue(Paragraph paragraph) {
        List<AttrValueVO> supplementValue = null;
        /* 生成报告时，段落的补充说明使用段落定制中配置的段落补充说明 */
        String customId = paragraph.getCustomId();
        ParagraphCustom paragraphCustom = paragraphCustomService.queryParagraphCustomDetail(customId);
        List<AttrDefineVO> customSupplement = Lists.newArrayList(paragraphCustom.getSupplement());
        if (!CollectionUtils.isEmpty(customSupplement)) {
            supplementValue = customSupplement.stream().map(attrDefineVO -> {
                AttrValueVO attrValueVO = new AttrValueVO();
                attrValueVO.setCode(attrDefineVO.getCode());
                attrValueVO.setValue(attrDefineVO.getDefaultValue());
                attrValueVO.setType(AttrValueTypeEnum.str);
                return attrValueVO;
            }).collect(Collectors.toList());

        }
        return supplementValue;
    }


    /**
     * 获取头部的属性值
     *
     * @param paragraphContentVO 段落内容属性值
     * @param reportOutline      报告概要对象，包含报告所属和报告月份的基本信息
     * @return 头部的属性值
     * @author lixing
     * @version V1.0 2021/7/21 5:24 下午
     */
    public abstract List<AttrValueVO> getHeader(
            ParagraphContentVO paragraphContentVO, ReportOutline reportOutline);

    /**
     * 计算分组列表和条目列表的加和值
     *
     * @param groupVOList 分组列表
     * @param itemVOList  条目列表
     * @param codeSet     要加和的属性集
     * @return 总计
     * @author lixing
     * @version V1.0 2021/7/21 2:32 下午
     */
    protected Map<String, AttrValueVO> getSum(List<GroupVO> groupVOList, List<ItemVO> itemVOList, Set<String> codeSet) {
        List<List<AttrValueVO>> valuesList = new ArrayList<>();
        // 将条目的属性集加入统计队列
        if (!CollectionUtils.isEmpty(itemVOList)) {
            List<List<AttrValueVO>> itemValuesList = itemVOList.stream().map(
                    ItemVO::getAttrs).collect(Collectors.toList());
            valuesList.addAll(itemValuesList);
        }
        // 将分组的总计加入统计队列
        if (!CollectionUtils.isEmpty(groupVOList)) {
            List<List<AttrValueVO>> groupSummaryList = groupVOList.stream().map(
                    GroupVO::getSummary).collect(Collectors.toList());
            valuesList.addAll(groupSummaryList);
        }

        if (CollectionUtils.isEmpty(valuesList)) {
            return null;
        }

        /* 筛选统计队列中每条数据需要加和的属性,将筛选后的属性编码和值放入map中存储 */
        List<Map<String, AttrValueVO>> values = new ArrayList<>();
        for (List<AttrValueVO> attrValues : valuesList) {
            if (CollectionUtils.isEmpty(attrValues)) {
                continue;
            }
            // 将统计队列中的一条数据转换为map
            Map<String, AttrValueVO> data = attrValueList2Map(codeSet, attrValues);
            values.add(data);
        }

        if (CollectionUtils.isEmpty(values) || CollectionUtils.isEmpty(codeSet)) {
            return null;
        }

        Map<String, AttrValueVO> result = new HashMap<>();
        /* 对统计队列中的number类型的值进行加和 */
        for (String code : codeSet) {
            BigDecimal attrSumValue = null;
            result.put(code, null);
            AttrValueVO codeAttrSumValue = null;
            for (Map<String, AttrValueVO> data : values) {
                String attrValue = data.get(code) == null ? null : data.get(code).getValue();
                if (StringUtils.isBlank(attrValue)) {
                    continue;
                }
                codeAttrSumValue = new AttrValueVO();
                BeanUtils.copyProperties(data.get(code), codeAttrSumValue);
                if (attrSumValue == null) {
                    attrSumValue = new BigDecimal("0");
                }
                attrSumValue = attrSumValue.add(new BigDecimal(attrValue));
            }
            if (attrSumValue != null) {
                codeAttrSumValue.setValue(attrSumValue.toString());
            }
            result.put(code, codeAttrSumValue);
        }

        return result;
    }

    /**
     * 将属性值对象列表转换为map code -> 属性值对象
     *
     * @param codeSet    需要转换的属性集
     * @param attrValues 原始数据
     * @return code -> 属性值对象的映射
     * @author lixing
     * @version V1.0 2021/7/26 2:16 下午
     */
    public static Map<String, AttrValueVO> attrValueList2Map(Set<String> codeSet, List<AttrValueVO> attrValues) {
        // 将统计队列中的一条数据转换为map
        Map<String, AttrValueVO> data = new HashMap<>();
        for (AttrValueVO attrValue : attrValues) {
            if (attrValue == null || !codeSet.contains(attrValue.getCode())) {
                continue;
            }
            codeSet.add(attrValue.getCode());
            data.put(attrValue.getCode(), attrValue);
        }
        return data;
    }

    /**
     * 键值对转换为属性对象列表
     *
     * @param codeValueMap 键值对
     * @return 属性对象列表
     * @author lixing
     * @version V1.0 2021/7/26 10:54 上午
     */
    protected List<AttrValueVO> map2ValueList(Map<String, AttrValueVO> codeValueMap) {
        List<AttrValueVO> results = new ArrayList<>();
        if (!CollectionUtils.isEmpty(codeValueMap)) {
            return codeValueMap.values().stream().
                    filter(Objects::nonNull).collect(Collectors.toList());
        }
        return results;
    }

    /**
     * 计算分组列表和条目列表的统计值
     *
     * @param groupVOList   分组列表
     * @param itemVOList    条目列表
     * @param projectIdList 项目id列表（统计可能需要项目的一些属性信息）
     * @return 总计
     * @author lixing
     * @version V1.0 2021/7/21 2:32 下午
     */
    public abstract List<AttrValueVO> getSummary(
            List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList);

    public List<AttrValueVO> initItem(ItemVO itemVO, String functionId) {
        itemVO.setId(functionId);
        Function function = functionService.queryFunctionDetail(functionId);
        itemVO.setName(function.getName());
        List<AttrValueVO> attrs = new ArrayList<>();
        itemVO.setAttrs(attrs);
        return attrs;
    }
}
