package com.persagy.apm.report.paragraphs.service.impl.energyusage;

import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.energyUsage.EnergyUsageContentCodes;
import com.persagy.apm.report.indicator.codes.energyUsage.EnergyUsageHeaderCodes;
import com.persagy.apm.report.indicator.factory.EnergyUsageFactory;
import com.persagy.apm.report.indicator.model.entity.EnergyUsageDefine;
import com.persagy.apm.report.indicator.utils.IndicatorUtils;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import com.persagy.apm.report.paragraphs.service.impl.ProjectParagraphBuilder;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * 能耗段落单项目建造者
 *
 * @author lixing
 * @version V1.0 2021/7/20 9:36 上午
 **/
@Service
public class ProjectEnergyUsageBuilder extends ProjectParagraphBuilder {
    @Autowired
    EnergyUsageFactory energyUsageFactory;
    @Autowired
    IFunctionService functionService;
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    IReportOutlineService reportOutlineService;

    @Override
    public List<AttrValueVO> getHeader(ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        List<String> relatedProjectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);

        List<AttrValueVO> summary = paragraphContentVO.getSummary();
        Set<String> codeSet = Sets.newSet(
                EnergyUsageContentCodes.currentMonth.name(),
                EnergyUsageContentCodes.chainRange.name(),
                EnergyUsageContentCodes.sametimeRange.name()
        );

        Map<String, AttrValueVO> data = attrValueList2Map(codeSet, summary);

        List<AttrValueVO> result = new ArrayList<>();
        // 总耗电量
        AttrValueVO currentMonth = data.get(EnergyUsageContentCodes.currentMonth.name());

        if (currentMonth != null) {
            AttrValueVO totalUsage = new AttrValueVO(
                    EnergyUsageHeaderCodes.totalUsage.name(), currentMonth.getValue(),
                    currentMonth.getType(), currentMonth.getUnit());
            result.add(totalUsage);
        }
        // 同比增幅
        AttrValueVO sametimeRange = data.get(EnergyUsageContentCodes.sametimeRange.name());
        if (sametimeRange != null) {
            result.add(new AttrValueVO(
                    EnergyUsageHeaderCodes.sameTimeRange.name(), sametimeRange.getValue(),
                    sametimeRange.getType(), sametimeRange.getUnit()));
        }
        // 环比增幅
        AttrValueVO chainRange = data.get(EnergyUsageContentCodes.chainRange.name());
        if (chainRange != null) {
            result.add(new AttrValueVO(
                    EnergyUsageHeaderCodes.chainRange.name(), chainRange.getValue(),
                    chainRange.getType(), chainRange.getUnit()));
        }
        // 同等碳排放量
        if (currentMonth != null) {
            AttrValueVO tce = energyUsageFactory.getTce(DataUtils.parseDouble(currentMonth.getValue()));
            result.add(tce);
        }
        // 总建筑面积
        Double totalArea = getProjectBuildingArea(relatedProjectIds);
        String totalAreaStr = totalArea == null ? null : String.valueOf(totalArea);
        result.add(new AttrValueVO(
                EnergyUsageHeaderCodes.totalArea.name(), totalAreaStr, AttrValueTypeEnum.number_1));
        // 商业面积
        Double projectCommercialArea = getProjectCommercialArea(relatedProjectIds);
        String projectCommercialAreaStr = projectCommercialArea == null ? null : String.valueOf(projectCommercialArea);
        result.add(new AttrValueVO(
                EnergyUsageHeaderCodes.commercialArea.name(), projectCommercialAreaStr, AttrValueTypeEnum.number_1));
        // 公区面积
        Double projectPublicArea = getProjectPublicArea(relatedProjectIds);
        String projectPublicAreaStr = projectPublicArea == null ? null : String.valueOf(projectPublicArea);
        result.add(new AttrValueVO(
                EnergyUsageHeaderCodes.publicArea.name(), projectPublicAreaStr, AttrValueTypeEnum.number_1));
        // 地下停车场面积
        Double projectUndergroundArea = getProjectUndergroundArea(relatedProjectIds);
        String projectUndergroundAreaStr = projectUndergroundArea == null ? null : String.valueOf(projectUndergroundArea);
        result.add(new AttrValueVO(
                EnergyUsageHeaderCodes.undergroundArea.name(), projectUndergroundAreaStr, AttrValueTypeEnum.number_1));
        return result;
    }

    @Override
    public List<AttrValueVO> getSummary(
            List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {

        Set<String> codeSet = Sets.newSet(
                EnergyUsageContentCodes.currentMonth.name(),
                EnergyUsageContentCodes.lastMonth.name(),
                EnergyUsageContentCodes.chainGrowth.name(),
                EnergyUsageContentCodes.sameTime.name(),
                EnergyUsageContentCodes.sametimeGrowth.name(),
                EnergyUsageContentCodes.aggregate.name(),
                EnergyUsageContentCodes.aggregateLastYear.name(),
                EnergyUsageContentCodes.aggregateGrowth.name()
        );

        Map<String, AttrValueVO> sum = super.getSum(groupVOList, itemVOList, codeSet);

        if (CollectionUtils.isEmpty(sum)) {
            return null;
        }

        List<AttrValueVO> results = super.map2ValueList(sum);
        /* 计算增幅 */
        // 环比增幅
        AttrValueVO chainRange = energyUsageFactory.getChainRange(
                IndicatorUtils.getAttrDoubleValue(
                        sum.get(EnergyUsageContentCodes.chainGrowth.name())
                ),
                IndicatorUtils.getAttrDoubleValue(
                        sum.get(EnergyUsageContentCodes.lastMonth.name())
                )
        );
        results.add(chainRange);
        // 同比增幅
        AttrValueVO sameTimeRange = energyUsageFactory.getSameTimeRange(
                IndicatorUtils.getAttrDoubleValue(
                        sum.get(EnergyUsageContentCodes.sametimeGrowth.name())
                ),
                IndicatorUtils.getAttrDoubleValue(
                        sum.get(EnergyUsageContentCodes.sameTime.name())
                )
        );
        results.add(sameTimeRange);
        // 累计同比增幅
        AttrValueVO aggregateRange = energyUsageFactory.getAggregateRange(
                IndicatorUtils.getAttrDoubleValue(sum.get(EnergyUsageContentCodes.aggregateGrowth.name())),
                IndicatorUtils.getAttrDoubleValue(sum.get(EnergyUsageContentCodes.aggregateLastYear.name())));
        results.add(aggregateRange);

        /* 计算单位面积信息 */
        Double projectBuildingArea = getProjectBuildingArea(projectIdList);
        // 单位面积用量
        AttrValueVO perSqm = energyUsageFactory.getPerSqm(
                IndicatorUtils.getAttrDoubleValue(sum.get(EnergyUsageContentCodes.currentMonth.name())),
                projectBuildingArea,
                IndicatorUtils.getAttrUnit(sum.get(EnergyUsageContentCodes.currentMonth.name()))
        );
        // 上月单位面积用量
        AttrValueVO perSqmLastMonth = energyUsageFactory.getPerSqmLastMonth(
                IndicatorUtils.getAttrDoubleValue(
                        sum.get(EnergyUsageContentCodes.lastMonth.name())
                ),
                projectBuildingArea,
                IndicatorUtils.getAttrUnit(sum.get(EnergyUsageContentCodes.lastMonth.name()))
        );
        // 去年同期单位面积用量
        AttrValueVO perSqmSameTime = energyUsageFactory.getPerSqmSameTime(
                IndicatorUtils.getAttrDoubleValue(sum.get(EnergyUsageContentCodes.perSqmSameTime.name())),
                projectBuildingArea,
                IndicatorUtils.getAttrUnit(sum.get(EnergyUsageContentCodes.perSqmSameTime.name()))
        );
        // 单位面积环比增量
        AttrValueVO perSqmChainGrowth = energyUsageFactory.getPerSqmChainGrowth(
                IndicatorUtils.getAttrDoubleValue(perSqm),
                IndicatorUtils.getAttrDoubleValue(perSqmLastMonth),
                IndicatorUtils.getAttrUnit(perSqm)
        );
        // 单位面积环比增幅
        AttrValueVO perSqmChainRange = energyUsageFactory.getPerSqmChainRange(
                IndicatorUtils.getAttrDoubleValue(perSqmChainGrowth),
                IndicatorUtils.getAttrDoubleValue(perSqmLastMonth)
        );
        // 单位面积同比增量
        AttrValueVO perSqmSameTimeGrowth = energyUsageFactory.getPerSqmSameTimeGrowth(
                IndicatorUtils.getAttrDoubleValue(perSqm),
                IndicatorUtils.getAttrDoubleValue(perSqmSameTime),
                IndicatorUtils.getAttrUnit(perSqm)
        );
        // 单位面积同比增幅
        AttrValueVO perSqmSameTimeRange = energyUsageFactory.getPerSqmSameTimeRange(
                IndicatorUtils.getAttrDoubleValue(perSqmSameTimeGrowth),
                IndicatorUtils.getAttrDoubleValue(perSqmSameTime)
        );
        results.add(perSqm);
        results.add(perSqmLastMonth);
        results.add(perSqmSameTime);
        results.add(perSqmChainGrowth);
        results.add(perSqmChainRange);
        results.add(perSqmSameTimeGrowth);
        results.add(perSqmSameTimeRange);

        return results;
    }

    @Override
    public ItemVO getProjectItemValue(String itemId, String projectId, Date reportMonth) {
        // 获取指标的定义，转换为指标对象
        Function function = functionService.queryFunctionDetail(itemId);
        Gson gson = new Gson();
        EnergyUsageDefine define;
        try {
            define = gson.fromJson(function.getDefine(), EnergyUsageDefine.class);
            if (define.getItemId() == null || define.getModelCode() == null) {
                throw new IllegalArgumentException("能耗用量定义配置错误，错误的定义内容为：" + function);
            }
        } catch (JsonSyntaxException e) {
            throw new IllegalArgumentException("能耗用量定义配置错误，错误的定义内容为：" + function);
        }
        // 从指标库中获取指标取值
        return energyUsageFactory.getEnergyUsageInfo(function, define, projectId, reportMonth);
    }

    @Override
    public List<AttrValueVO> getProjectSummary(ParagraphContentVO projectParagraphContent) {
        if (projectParagraphContent == null) {
            return null;
        }
        return projectParagraphContent.getSummary();
    }

    /**
     * 获取项目面积总和
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 项目面积总和
     * @author lixing
     * @version V1.0 2021/7/26 9:37 上午
     */
    public Double getProjectBuildingArea(List<String> projectIds) {
        return saasWebService.getTotalBuildingArea(projectIds);
    }

    /**
     * 获取项目商业面积总和
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 项目商业面积总和
     * @author lixing
     * @version V1.0 2021/7/26 9:37 上午
     */
    public Double getProjectCommercialArea(List<String> projectIds) {
        return saasWebService.getTotalCommercialArea(projectIds);
    }

    /**
     * 获取项目公区面积总和
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 项目公区面积总和
     * @author lixing
     * @version V1.0 2021/7/26 9:37 上午
     */
    public Double getProjectPublicArea(List<String> projectIds) {
        return saasWebService.getTotalPublicArea(projectIds);
    }

    /**
     * 获取项目地下停车场面积总和
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 项目地下停车场面积总和
     * @author lixing
     * @version V1.0 2021/7/26 9:37 上午
     */
    public Double getProjectUndergroundArea(List<String> projectIds) {
        return saasWebService.getTotalUndergroundArea(projectIds);
    }
}
